@@ -1,4 +1,5 @@
 import json
+from typing import List
 from copy import deepcopy
 from urllib.parse import urlparse
 
@@ -8,7 +9,8 @@ from .exceptions import (
     ClientNotFoundError,
     MediaNotFound,
 )
-from .extractors import extract_media_v1, extract_media_gql, extract_user_short
+from .extractors import extract_media_v1, extract_media_gql, extract_comment, extract_media_oembed
+from .types import Usertag, Location, UserShort
 
 
 class Media:
@@ -24,7 +26,7 @@ class Media:
                 "media_id must been contain digits, now %s" % media_id
             )
             user = self.media_user(media_id)
-            media_id = "%s_%s" % (media_id, user["pk"])
+            media_id = "%s_%s" % (media_id, user.pk)
         return media_id
 
     @staticmethod
@@ -84,12 +86,10 @@ class Media:
         )
         if not data.get("shortcode_media"):
             raise MediaNotFound(media_pk=media_pk, **data)
+        data['shortcode_media']['location'] = self.location_complete(
+            data['shortcode_media']['location']
+        )
         media = extract_media_gql(data["shortcode_media"])
-        location = media.get('location')
-        if location:
-            location_pk = location.get('pk')
-            if not location.get('lat') and location_pk:
-                media['location'] = self.location_info(location_pk)
         return media
 
     def media_info_v1(self, media_pk: int) -> dict:
@@ -134,7 +134,14 @@ class Media:
         self._medias_cache.pop(self.media_pk(media_id), None)
         return result.get("did_delete")
 
-    def media_edit(self, media_id: str, caption: str, title: str = "", usertags: list = [], location: dict = {}) -> bool:
+    def media_edit(
+        self,
+        media_id: str,
+        caption: str,
+        title: str = "",
+        usertags: List[Usertag] = [],
+        location: Location = None
+    ) -> dict:
         """Edit caption for media
         Example: https://i.instagram.com/api/v1/media/2154602296692269830_1903424587/edit_media/
 
@@ -166,7 +173,7 @@ class Media:
         media_id = self.media_id(media_id)
         media = self.media_info(media_id)  # from cache
         usertags = [
-            {"user_id": tag['user']['pk'], "position": tag['position']}
+            {"user_id": tag.user.pk, "position": [tag.x, tag.y]}
             for tag in usertags
         ]
         data = {
@@ -177,7 +184,7 @@ class Media:
             "usertags": json.dumps({"in": usertags}),
             "is_carousel_bumped_post": "false",
         }
-        if media["product_type"] == "igtv":
+        if media.product_type == "igtv":
             if not title:
                 try:
                     title, caption = caption.split("\n", 1)
@@ -194,48 +201,23 @@ class Media:
         )
         return result
 
-    def media_user(self, media_pk: int) -> dict:
+    def media_user(self, media_pk: int) -> UserShort:
         """Get user object
         """
         # return extract_user_short(
         #     self._media_info_a1(InstagramIdCodec.encode(media_pk))["owner"]
         # )
-        return self.media_info(media_pk)["user"]
+        return self.media_info(media_pk).user
 
     def media_oembed(self, url: str) -> dict:
         """Return info about media and user by post URL
-        Example: https://i.instagram.com/api/v1/oembed/?
-            url=https://instagram.com/p/B1LbfVPlwIA?
-            ig_mid=D68F44BF-2EDC-43AF-BB7F-D693BA0ABF05&
-            utm_source=instagramweb
-        Result: {
-            "version": "1.0",
-            "title": "",
-            "author_name": "mind__flowers",
-            "author_url": "https://www.instagram.com/mind__flowers",
-            "author_id": 8572539084,
-            "media_id": "2110901750722920960_8572539084",
-            "provider_name": "Instagram",
-            "provider_url": "https://www.instagram.com",
-            "type": "rich",
-            "width": 658,
-            "height": null,
-            "html": '...",
-            "thumbnail_width": 640,
-            "thumbnail_height": 799,
-            "can_view": true,
-        }
         """
-        return self.private_request(f"oembed?url={url}")
+        return extract_media_oembed(
+            self.private_request(f"oembed?url={url}")
+        )
 
     def media_comments(self, media_id: str) -> list:
         """Get list of comments for media
-        Example: https://i.instagram.com/api/v1/media/2277659671519488169_8530598273/comments/?
-        inventory_source=media_or_ad&
-        analytics_module=comments_v2_feed_timeline&
-        can_support_threading=true&
-        is_carousel_bumped_post=false&
-        feed_position=0
         """
         # TODO: to public or private
         media_id = self.media_id(media_id)
@@ -247,8 +229,7 @@ class Media:
                     f"media/{media_id}/comments/", params={"max_id": max_id}
                 )
                 for comment in result["comments"]:
-                    comment["user"] = extract_user_short(comment["user"])
-                    comments.append(comment)
+                    comments.append(extract_comment(comment))
                 if not result["has_more_comments"]:
                     break
                 max_id = result["next_max_id"]
@@ -262,19 +243,6 @@ class Media:
 
     def media_comment(self, media_id: str, text: str) -> int:
         """Comment media
-        Example: {
-            "user_breadcrumb": "u3QwgWYbhXMTf7nKhvpBYMyWxI9IScUmtUJ69TeTVwY=\nNCAxNDg5IDAgMTU4NjE5Mjg3MzI4NA==\n",
-            "delivery_class": "organic",
-            "idempotence_token": "9d68650c-bb63-4c09-a367-53d0bc177de4",
-            "_csrftoken": "H8Rk6Ry2ffWcUSwWIBblVK4hHHII2RMk",
-            "radio_type": "wifi-none",
-            "_uid": "8530598273",
-            "_uuid": "c642fece-8663-40d8-8ab7-112df0179e65",
-            "comment_text": "Test",
-            "is_carousel_bumped_post": "false",
-            "container_module": "self_comments_v2_feed_contextual_self_profile",
-            "feed_position": "0"
-        }
         """
         assert self.user_id, "Login required"
         media_id = self.media_id(media_id)
@@ -291,6 +259,4 @@ class Media:
                 }
             ),
         )
-        comment = result["comment"]
-        comment["user"] = extract_user_short(comment["user"])
-        return comment
+        return extract_comment(result["comment"])
